package sun.reflect.annotation;

import sun.misc.SharedSecrets;
import java.lang.annotation.AnnotationFormatError;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Objects;
import java.util.Iterator;
import java.lang.annotation.Repeatable;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.ArrayList;
import java.lang.annotation.Annotation;
import java.util.Map;
import sun.misc.JavaLangAccess;

public final class AnnotationSupport
{
    private static final JavaLangAccess LANG_ACCESS;
    
    public AnnotationSupport() {
    }
    
    public static <A extends Annotation> A[] getDirectlyAndIndirectlyPresent(Map<Class<? extends Annotation>, Annotation> map, Class<A> componentType) {
        ArrayList<Annotation> list = new ArrayList();
        Annotation annotation = (Annotation)map.get(componentType);
        if (annotation != null) {
            list.add(annotation);
        }
        Annotation[] indirectlyPresent = getIndirectlyPresent(map, componentType);
        if (indirectlyPresent != null && indirectlyPresent.length != 0) {
            list.addAll((annotation == null || containerBeforeContainee(map, componentType)) ? 0 : 1, Arrays.asList(indirectlyPresent));
        }
        return (Annotation[])list.toArray((Annotation[])(Annotation[])Array.newInstance(componentType, list.size()));
    }
    
    private static <A extends Annotation> A[] getIndirectlyPresent(Map<Class<? extends Annotation>, Annotation> map, Class<A> clazz) {
        Repeatable repeatable = (Repeatable)clazz.getDeclaredAnnotation(Repeatable.class);
        if (repeatable == null) {
            return null;
        }
        Annotation annotation = (Annotation)map.get(repeatable.value());
        if (annotation == null) {
            return null;
        }
        Annotation[] valueArray = getValueArray(annotation);
        checkTypes(valueArray, annotation, clazz);
        return valueArray;
    }
    
    private static <A extends Annotation> boolean containerBeforeContainee(Map<Class<? extends Annotation>, Annotation> map, Class<A> clazz) {
        Class<Object> value = ((Repeatable)clazz.getDeclaredAnnotation(Repeatable.class)).value();
        for (Class clazz2 : map.keySet()) {
            if (clazz2 == value) {
                return true;
            }
            if (clazz2 == clazz) {
                return false;
            }
            continue;
        }
        return false;
    }
    
    public static <A extends Annotation> A[] getAssociatedAnnotations(Map<Class<? extends Annotation>, Annotation> map, Class<?> obj, Class<A> clazz) {
        Objects.requireNonNull(obj);
        Annotation[] array = getDirectlyAndIndirectlyPresent(map, clazz);
        if (AnnotationType.getInstance(clazz).isInherited()) {
            for (Class clazz2 = obj.getSuperclass(); array.length == 0 && clazz2 != null; array = getDirectlyAndIndirectlyPresent(AnnotationSupport.LANG_ACCESS.getDeclaredAnnotationMap(clazz2), clazz), clazz2 = clazz2.getSuperclass()) {}
            return array;
        }
        return array;
    }
    
    private static <A extends Annotation> A[] getValueArray(Annotation obj) {
        try {
            AnnotationType instance = AnnotationType.getInstance(obj.annotationType());
            if (instance == null) {
                throw invalidContainerException(obj, null);
            }
            Method method = (Method)instance.members().get("value");
            if (method == null) {
                throw invalidContainerException(obj, null);
            }
            method.setAccessible(true);
            return (Annotation[])(Annotation[])method.invoke(obj, new Object[0]);
        }
        catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException | ClassCastException ex) {
            throw invalidContainerException(obj, ex);
        }
    }
    
    private static AnnotationFormatError invalidContainerException(Annotation obj, Throwable cause) {
        return new AnnotationFormatError(obj + " is an invalid container for repeating annotations", cause);
    }
    
    private static <A extends Annotation> void checkTypes(A[] array, Annotation annotation, Class<A> clazz) {
        int length = array.length;
        int i = 0;
        while (i < length) {
            if (!clazz.isInstance(array[i])) {
                throw new AnnotationFormatError(String.format("%s is an invalid container for repeating annotations of type: %s", new Object[] { annotation, clazz }));
            }
            ++i;
        }
    }
    
    static {
        LANG_ACCESS = SharedSecrets.getJavaLangAccess();
    }
}
